{------------------------------------------------------------------------------}
{
   Unit Name: FormCommands
   Purpose  : An simple "add in macro" language for applications
   Author   : Vesa Lappalainen
   Date     : 1999 - 26.11.2000
   Changed  :

   ToDo     :

   Usage    :

   Syntax for macro programs:
      [Title]
      Title = Sopivan tenttiajan kysely
      [Params]
      [Program]
      0=DEBUG 1
      1=OPEN tmp\ajatvko4851.txt
      2=SAVEAS tentit.jpg
      3=CLOSE

    1) In "main form" add program commands

        Programs := TParamProgram.Create(self);
        Programs.Add(TProgramCommand.Create('OPEN',StepOPEN,'datafile'));
        Programs.Add(TProgramCommand.Create('SAVEAS',StepSAVEAS,'JPG_filename'));
        Programs.Add(TProgramCommand.Create('EXEC',StepEXEC,''));
        Programs.Add(TProgramCommand.Create('CLOSE',StepCLOSE));

    2) Implement command on "main form", Result is for error strings

          function TFormTimeTables.StepOPEN(const s:string):string;
          begin
            EditFileName.Text := s;
            ReadFile;
            Result := '';
          end;

          function TFormTimeTables.StepSAVEAS(const s:string):string;
          begin
            FImageName := s;
            ActionFileSaveExecute(self);
            Result := '';
          end;

          function TFormTimeTables.StepEXEC(const s:string):string;
          begin
            ReadFile;
            Result := '';
          end;

          function TFormTimeTables.StepCLOSE(const s:string):string;
          begin
            Close;
            Result := '';
          end;

    3) Start program somewhere, for example make a function for
       that:
          function TFormTimeTables.CheckProgram : boolean;
          var s:string;
          begin
            Result := false;
            if ParamCount <= 2 then exit;
            if ( ParamStr(1) <> 'RUN' ) then exit;
            s := Programs.Read(ParamStr(2));
            if ( s <> '' ) then begin ShowMessage(s); exit; end;
            Programs.Run;
            Result := true;
          end;

       You can also add a button or menu to load and run programs.

    4) Call CheckPrograms in mainprogram just before
          Application.Run;

    5) Properties and methods:

    procedure Add(command:TProgramCommand);
      - to add new program command.  The most usual way to use is to call
          constructor TProgramCommand.Create(const name:string;
               command:TProgramCommandFunction;const syntax:string='');

    function Read(filename:string) : string;
      - read the program from file

    procedure Run;
      - start to run the program

    function Execute(const s:string):string;
      - execute one program command

    procedure SaveParams(const name:string);
      - save parametrs to file

    procedure ShowProgramCommands;
      - show and Form with all possible commands

    procedure Stop;
      - stop the program run

  published
    property Title:TPanel read FTitle write SetTitle;
      - pointer to the panel, where the Title of the program can be
        written

    property TitleStr : string read FTitleStr write SetTitleStr;
      - to change the programs TitleStr

    property EditWidth : integer read FEditWidth write SetEditWidth default 100;
    property EditHeight : integer read FEditHeight write SetEditHeight;
      - to set the parameter input box width and height for all paraneters,
        see ParamCtr

    property ParamCtr : TWinControl read FParamCtr write FParamCtr;
      - the home for possible params on the form
        Params are introduced in macro program code like:
          [Params]
          Start date  = 199901
          End date    = 200013
          Date pos    = 1 for 6
        and they are used like:
          [Program]
          1=SQL select substring(dtime from [Date pos]) dtime,cut,material,color from rsheet where (dtime>="[Start date]") and (dtime<="[End date]")
          ...
        From every param an Param-input box is generated inside ParamCtr

    property Debug : integer read FDebug write FDebug default 1;
      - to change the debug level
          0 = no debug
          1 = status is shown in possible Title-panel
          2 = every step is confirmed by MessageDlg

    property Help : string read getHelp;
      - gives a list of all command separated by cr/lf.


}
{------------------------------------------------------------------------------}

unit FormCommands;

interface
uses Forms,SysUtils,Classes, Controls,StdCtrls,ExtCtrls;

type
  TProgramCommandFunction = function (const s:string):string of object;

  TProgramCommand = class
  private
    function getHelp: string;
  public
    name : string;
    Command : TProgramCommandFunction;
    syntax:string;
    constructor Create(const name:string; command:TProgramCommandFunction;const syntax:string=''); virtual;
    function Execute(const s:string):string; virtual;
  published
    property Help:string read getHelp;
  end;

  TProgramCommands = class
  private
    flist : TStrings;
    function getHelp: string;
  public
    constructor Create; overload;
    destructor Destroy; override;
    procedure Add(command:TProgramCommand);
    function Execute(const name,s:string):string;
  published
    property Help : string read getHelp;
  end;

  TParam = class(TPanel)
  private
    FEdit : TEdit;
    function GetValue: string;
    procedure SetValue(const Value: string);
    procedure SetEditWidth(const Value: integer);
    function GetEditWidth: integer;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Value : string read GetValue write SetValue;
    property EditWidth : integer read GetEditWidth write SetEditWidth;
  end;

  TParamProgram = class(TComponent)
  private
    FCommands : TProgramCommands;
    FTitle : TPanel;
    FStatus : TLabel;
    FTitleStr : string;
    FParamCtr : TWinControl;
    FEditWidth : integer;
    FEditHeight : integer;
    FLines : TStrings;
    FDebug : integer;
    FStop : boolean;
    procedure SetTitle(const Value: TPanel);
    procedure SetTitleStr(const Value: string);
    procedure SetEditWidth(const Value: integer);
    procedure SetEditHeight(const Value: integer);
    function FindParam(name: string): TParam;
    function SetDebug(const s:string):string;
    procedure HideParams;
    procedure ReplaceParams(var line: string);
    function getHelp: string;
    procedure StopProgram(Sender: TObject);
    procedure CloseShowCommands(sender:TObject;  var Action: TCloseAction);
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure Add(command:TProgramCommand);
    function Read(filename:string) : string;
    procedure Run;
    function Execute(const s:string):string;
    procedure SaveParams(const name:string);
    procedure ShowProgramCommands;
    procedure Stop;
  published
    property Title:TPanel read FTitle write SetTitle;
    property TitleStr : string read FTitleStr write SetTitleStr;
    property EditWidth : integer read FEditWidth write SetEditWidth default 100;
    property EditHeight : integer read FEditHeight write SetEditHeight;
    property ParamCtr : TWinControl read FParamCtr write FParamCtr;
    property Debug : integer read FDebug write FDebug default 1;
    property Help : string read getHelp;
  end;


implementation
uses IniFiles,Dialogs;

procedure Replace(var s:string; const what,by:string);
var maxl,oldp,p,l:integer;
begin
  oldp := 0;
  l := Length(what);
  maxl := Length(s)*Length(by);
  while true do begin
    p := Pos(what,s);
    if ( p <= 0 ) then exit;
    if ( p = oldp ) then exit;        // Care for recursion
    Delete(s,p,l);
    Insert(by,s,p);
    if ( Length(s) > maxl ) then exit; // Care for recursion
  end;
end;

procedure Split2trim(const s,by:string; var s1,s2:string);
var p:integer;
begin
  p := pos(by,s);
  s1 := s;
  s2 := '';
  if ( p > 0 ) then begin
    Delete(s1,p,10000);
    s2 := copy(s,p+1,10000);
  end;
  s1 := Trim(s1);
  s2 := Trim(s2);
end;


{ TProgramCommand }

constructor TProgramCommand.Create(const name: string;
            command: TProgramCommandFunction; const syntax:string);
begin
  inherited Create;
  self.name := name;
  self.Command := command;
  self.Syntax := syntax;
end;

function TProgramCommand.Execute(const s: string): string;
begin
  Result := '';
  if ( @Command = nil ) then exit;
  Result := Command(s);
end;

function TProgramCommand.getHelp: string;
begin
  Result := name + ' ' + syntax;
end;


{ TProgramCommands }

procedure TProgramCommands.Add(command: TProgramCommand);
begin
  if ( command = nil ) then exit;
  flist.AddObject(command.name,command);
end;

constructor TProgramCommands.Create;
begin
  inherited;
  flist := TStringList.Create;
end;

destructor TProgramCommands.Destroy;
var i:integer;
begin
  for i:=0 to flist.Count-1 do
    flist.Objects[i].Free;
  flist.free;
  inherited;
end;

function TProgramCommands.Execute(const name,s: string): string;
var i:integer;
begin
  i := flist.IndexOf(name);
  if ( i >= 0 ) then begin
    Result := TProgramCommand(flist.Objects[i]).Execute(s);
    exit;
  end;
  Result := 'Command not found "'+name+'"';
end;

function TProgramCommands.getHelp: string;
var i:integer;
begin
  Result := '';
  for i:=0 to flist.Count-1 do 
    Result := Result + TProgramCommand(flist.Objects[i]).Help + #13#10;
end;

{ TParam }

constructor TParam.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alTop;
  Top := 10000;
  BevelInner := bvLowered;
  Alignment := taLeftJustify;
  self.Height := height;
  FEdit := TEdit.Create(self);
  FEdit.Parent := self;
  FEdit.Width := 100;
  FEdit.Height := height;
  FEdit.Top := 0;
  FEdit.Left := self.width - FEdit.Width;
  FEdit.Anchors := [akRight,akTop];
  FEdit.Text := '';
end;

function TParam.GetEditWidth: integer;
begin
  Result := FEdit.Width;
end;

function TParam.GetValue: string;
begin
  Result := FEdit.Text;
end;


procedure TParam.SetEditWidth(const Value: integer);
begin
  FEdit.Width := Value;
  FEdit.Left := self.width - FEdit.Width;
  FEdit.Anchors := [akRight,akTop];
end;

procedure TParam.SetValue(const Value: string);
begin
  FEdit.Text := Value;
end;

{ TParamProgram }

procedure TParamProgram.Add(command: TProgramCommand);
begin
  if ( FCommands = nil ) then exit;
  FCommands.Add(command);
end;

constructor TParamProgram.Create(AOwner: TComponent);
begin
  inherited;
  FCommands := TProgramCommands.Create;
  Flines := TStringList.Create;
  FCommands.Add(TProgramCommand.Create('DEBUG',SetDebug,'[level] (0=no,1=status,2=full)'));
  FStatus := TLabel.Create(self);
  FStatus.OnClick := StopProgram;
  FStatus.Align := alClient;
  FStatus.AutoSize := false;
  FStatus.Visible := false;
  FStatus.ParentFont := false;
  FDebug := 1; // Show status
end;

destructor TParamProgram.Destroy;
begin
  FLines.Free;
  inherited;
end;

function TParamProgram.Execute(const s: string): string;
var command,value:string;
begin
  Split2trim(s,' ',command,value);
  Result := FCommands.Execute(command,value);
end;

procedure TParamProgram.HideParams;
var ip : integer; ctr : TComponent;
begin
  for ip := 0 to ComponentCount-1 do begin
    ctr := Components[ip];
    if ( not (ctr is TParam) ) then continue;
    TParam(ctr).Visible := false;
  end;
end;

function TParamProgram.FindParam(name:string):TParam;
var i : integer; ctr : TComponent; param : TParam;
begin
  Result := nil;
  for i := 0 to ComponentCount-1 do begin
    ctr := Components[i];
    if ( not (ctr is TParam) ) then continue;
    param := TParam(ctr);
    if ( param.Caption <> name ) then continue;
    Result := param;
    Exit;
  end;
end;

function TParamProgram.Read(filename: string) : string;
var prg:TIniFile; i:integer; dir,value,name:string; param:TParam;
    params : TStrings;
begin
  dir := ExtractFileDir(filename);
  if ( dir = '' ) then dir:='.\' else dir := '';
  prg := TIniFile.Create(dir+filename);
  Result := 'No programs ' + filename;
  if ( prg = nil ) then exit;
  params := TStringList.Create;
  TitleStr  := prg.ReadString('Title','Title','');
  FLines.Clear;
  prg.ReadSectionValues('Program',FLines);
  HideParams;
  prg.ReadSectionValues('Params',params);
  for i:=0 to params.Count-1 do begin
    split2trim(Params[i],'=',name,value);
    if ( name = '' ) then continue;
    param := FindParam(name);
    if ( param <> nil ) then begin
      param.Visible := true;
      param.Top := 10000;
      continue;
    end;
    param := TParam.Create(self);
    param.Height := EditHeight;
    param.EditWidth := EditWidth;
    param.Parent := ParamCtr;
    param.Caption := name;
    param.Value := value;
  end;
  prg.Free;
  if ( FLines.Count > 0 ) then Result := '';
end;

procedure TParamProgram.ReplaceParams(var line:string);
var ctr : TComponent; param : TParam; i:integer;
begin
  for i := 0 to ComponentCount-1 do begin
    ctr := Components[i];
    if ( not (ctr is TParam) ) then continue;
    param := TParam(ctr);
    Replace(line,'['+param.Caption+']',param.Value);
  end;
end;

procedure TParamProgram.Run;
procedure DoRun;
var p,i:integer; err,line:string;
    ret,retd : word;
begin
  ret := mrOK;
  for i := 0 to FLines.Count-1 do begin
    line := FLines[i];
    p := pos('=',line);
    if ( p > 0 ) then Delete(line,1,p);
    ReplaceParams(line);

    if ( FDebug > 1 ) then begin
      retd := MessageDlg(FLines[i]+#13#10+line+#13#10+'Continue?',
                         mtInformation,[mbOK,mbCancel,mbAll],0);
      if ( retd = mrCancel ) then exit;
      if ( retd = mrAll ) then FDebug := 1;
    end;

    if ( FStatus <> nil ) and ( FDebug > 0 ) then begin
      FStatus.Visible := true;
      FStatus.Caption := line;
      Application.ProcessMessages;
    end;

    if ( FStop ) then Exit;

    try
      err := Execute(line);
    except on e:Exception do
      err := e.Message;
    end;

    if ( err <> '' ) and ( ret <> mrAll ) then begin
      ret := MessageDlg(err+#13#10+'Continue?',mtError,[mbOK,mbCancel,mbAll],0);
      if ( ret = mrCancel ) then exit;
    end
  end;
end;
begin
  FStop := false;
  DoRun;
  if ( FStatus <> nil ) then FStatus.Visible := false;
end;

procedure TParamProgram.SetEditHeight(const Value: integer);
begin
  FEditHeight := Value;
end;

procedure TParamProgram.SetEditWidth(const Value: integer);
begin
  FEditWidth := Value;
end;

procedure TParamProgram.SetTitle(const Value: TPanel);
begin
  FTitle := Value;
  TitleStr := TitleStr;
  FStatus.Parent := Title;
end;

procedure TParamProgram.SetTitleStr(const Value: string);
begin
  FTitleStr := Value;
  if ( FTitle <> nil ) then FTitle.Caption := Value;
end;

function TParamProgram.SetDebug(const s: string): string;
begin
  Result := '';
  FDebug := StrToIntDef(s,0);
end;

procedure TParamProgram.SaveParams(const name: string);
var ip : integer; ctr : TComponent; param : TParam;
    prg:TIniFile;
    ident,value:string;
begin
  prg := TIniFile.Create(name);
  if ( prg = nil ) then exit;
  for ip := 0 to ComponentCount-1 do begin
    ctr := Components[ip];
    if ( not (ctr is TParam) ) then continue;
    param := TParam(ctr);
    if ( not param.Visible ) then continue;
    ident := param.Caption;
    value := param.Value;
    prg.WriteString('Params',ident,value);
  end;
  prg.Free;
end;

function TParamProgram.getHelp: string;
begin
  result := FCommands.Help;
end;

procedure TParamProgram.CloseShowCommands(sender:TObject;  var Action: TCloseAction);
begin
  TForm(sender).Release;
end;

procedure TParamProgram.ShowProgramCommands;
var f:TForm; l:TMemo;
begin
  try
    f := TForm.Create(nil);
    f.Caption := 'Program commands';
    f.OnClose := CloseShowCommands;
    l := TMemo.Create(f);
    l.Parent := f;
    l.Align := alClient;
    l.Text := FCommands.Help;
    f.Show;
  except end;
end;


procedure TParamProgram.Stop;
begin
  FStop := true;
end;

procedure TParamProgram.StopProgram(Sender: TObject);
begin
  Stop;
end;

end.

