{------------------------------------------------------------------------------}
{
   Unit Name: TimeTableForm
   Purpose  : Read and show students timetables
   Author   : Vesa Lappalainen
   Date     : 12.09.00
   Changed  : 18.11.00/vl
    + weeks
    + copy distribution to clipboard
   Changed  : 25.11.00/vl
    + save option
    + menus
    + programming languge to automate things (see FormCommands.pas)

   To compile: To project:  TimeTableForm (.pas, .dfm)
                            SubjectsForm (.pas, .dfm)
               Other files: files from kcomp\ needed


   ToDo     :

   Format of file:
     email=jaha%40st.jyu.fi&ma0810=&ti0810=&ke0810=TJT+C35&to0810=
     email=olhokkan@cc.jyu.fi&ma0810=&ti0810=TIE210&ke0810=&to0810=
   or:
     Weeks=48-51
     email=jaha%40st.jyu.fi&vko=48&ma0810=&ti0810=&ke0810=TJT+C35&to0810=
     email=olhokkan@cc.jyu.fi&vko=48&ma0810=&ti0810=TIE210&ke0810=&to0810=
}
{------------------------------------------------------------------------------}

unit TimeTableForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, StringGridIni, ComCtrls, ExtCtrls, kComp, KParam,
  kinicomp, savepos, ActnList, ExtDlgs, Menus, FormCommands;

type
  TOneWeekTable = class(TTabSheet)
  private
    Grid: TStringGridIni;
    FEmpty: boolean;
  protected
    procedure SetAsString(const Value: string); virtual;
    procedure SetEmpty(const Value: boolean);   virtual;
    function GetCol: integer;                   virtual;
    function GetRow: integer;                   virtual;
    procedure SetCol(const Value: integer);     virtual;
    procedure SetRow(const Value: integer);     virtual;
    function GetCells(c, r: integer): string;   virtual;
    procedure SetCells(c, r: integer; const Value: string); virtual;
  public
    constructor Create(AOwner:TComponent); override;
    property AsString : string write SetAsString;
    property Empty : boolean read FEmpty write SetEmpty;
    property Col : integer read GetCol write SetCol;
    property Row : integer read GetRow write SetRow;
    property Cells[c,r:integer]:string read GetCells write SetCells;
  end;

  TTimeTable = class(TTabSheet)
  private
    PanelText: TPanel;
    FEmail: TEdit;
    Rest : TLabel;
    FWeekPages : TPageControl;
    FFirstWeek : integer;
    FLastWeek : integer;
    FOnBeforeDraw : TDrawCellEvent;
    FOnDblClick : TNotifyEvent;
    FOnChange: TNotifyEvent;
  protected
    procedure SetAsString(const Value: string);                     virtual;
    function GetCells(w, c, r: integer): string;                    virtual;
    function GetWeekEmpty(w: integer): boolean;                     virtual;
    procedure SetFirstWeek(const Value: integer);                   virtual;
    procedure SetLastWeek(const Value: integer);                    virtual;
    procedure SetWeek(const Value: integer);                        virtual;
    function GetValid(w: integer): boolean;                         virtual;
    procedure SetCells(w, c, r: integer; const Value: string);      virtual;
    function GetCol: integer;                                       virtual;
    function GetRow: integer;                                       virtual;
    procedure SetCol(const Value: integer);                         virtual;
    procedure SetRow(const Value: integer);                         virtual;
    function GetWeeks(w: integer): TOneWeekTable;                   virtual;
    procedure SetOnBeforeDraw(const Value: TDrawCellEvent);         virtual;
    procedure SetOnDblClick(const Value: TNotifyEvent);             virtual;
    function GetWeek: integer;                                      virtual;
    procedure SetOnChange(const Value: TNotifyEvent);               virtual;
    function GetAsInteger(w, c, r: integer): integer;               virtual;
    procedure SetAsInteger(w, c, r: integer; const Value: integer); virtual;
    procedure SetWeekEmpty(w: integer; const Value: boolean);       virtual;
    function GetEMail: string;                                      virtual;
    procedure SetEMail(const Value: string);                        virtual;
  public
    constructor Create(AOwner:TComponent); overload; override;
    constructor Create(AOwner:TComponent;pg:TPageControl;fw,lw:integer); reintroduce; overload; virtual;
    property AsString : string write SetAsString;
    property Cells[w,c,r:integer] : string read GetCells write SetCells; default;
    property AsInteger[w,c,r:integer] : integer read GetAsInteger write SetAsInteger;
    property WeekEmpty[w:integer]:boolean read GetWeekEmpty write SetWeekEmpty;
    property Valid[w:integer]:boolean read GetValid;
    property Weeks[w:integer] : TOneWeekTable read GetWeeks;
    property FirstWeek : integer read FFirstWeek write SetFirstWeek;
    property LastWeek : integer read FLastWeek write SetLastWeek;
    property Week : integer read GetWeek write SetWeek;
    property Col : integer read GetCol write SetCol;
    property Row : integer read GetRow write SetRow;
    property OnBeforeDraw : TDrawCellEvent read FOnBeforeDraw write SetOnBeforeDraw;
    property OnDblClick : TNotifyEvent read FOnDblClick write SetOnDblClick;
    property OnChange: TNotifyEvent read FOnChange write SetOnChange;
    property eMail : string read GetEMail write SetEMail;
  end;

  TFormTimeTables = class(TForm)
    SavePos1: TSavePos;
    ActionList1: TActionList;
    ActionEditCopy: TAction;
    PanelButtons: TPanel;
    PanelOKButton: TPanel;
    ButtonOK: TButton;
    ButtonRead: TButton;
    EditFileName: TEditIni;
    ButtonAll: TButton;
    ParamBlackLimit: TkParam;
    CounterAnswers: TCounter;
    ParamFirstWeek: TkParam;
    ParamLastWeek: TkParam;
    PanelTimes: TPanel;
    PGTimes: TPageControl;
    TSAllHold: TTabSheet;
    PanelText: TPanel;
    LabelEmail: TLabel;
    WeekPagesHold: TPageControl;
    Tab1Hold: TTabSheet;
    Grid: TStringGridIni;
    CBReadWeeks: TCheckBoxIni;
    ActionFileSave: TAction;
    SavePictureDialog1: TSavePictureDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Edit1: TMenuItem;
    Save1: TMenuItem;
    Save2: TMenuItem;
    ActionFileExit: TAction;
    Exit1: TMenuItem;
    ActionFileOpen: TAction;
    OpenDialog1: TOpenDialog;
    Open1: TMenuItem;
    N1: TMenuItem;
    ActionShowProgramCommands: TAction;
    procedure ButtonReadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure GridBeforeDraw(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure ParamBlackLimitAfterChange(Sender: TObject; var val: Double);
    procedure GridDblClick(Sender: TObject);
    procedure ButtonOKClick(Sender: TObject);
    procedure ButtonAllClick(Sender: TObject);
    procedure WeekPagesChange(Sender: TObject);
    procedure ActionEditCopyExecute(Sender: TObject);
    procedure ActionFileSaveExecute(Sender: TObject);
    procedure ActionFileExitExecute(Sender: TObject);
    procedure ActionFileOpenExecute(Sender: TObject);
    procedure ActionShowProgramCommandsExecute(Sender: TObject);
  private
    TabAll: TTimeTable;
    FImageName : string;
    Programs : TParamProgram;
    procedure CountTimes;
    function FindTable(const s: string): TTimeTable;
    function GetWeek: integer;
    function GetFirstWeek: integer;
    function GetLastWeek: integer;
    procedure SetFirstWeek(const Value: integer);
    procedure SetLastWeek(const Value: integer);
    procedure SetWeek(const Value: integer);
    function IsWeeks(word,s: string): boolean;
    function StartsWith(const word, s: string): boolean;
    function GetImageBitmap: TBitmap;
    function StepCOPY(const s: string): string;
    function StepEXEC(const s: string): string;
    function StepOPEN(const s: string): string;
    function StepSAVEAS(const s: string): string;
    function StepCLOSE(const s: string): string;
    procedure ReadFile;
  public
    procedure ClearAll;
    procedure Choose(i:integer);
    function CheckProgram: boolean;
    property Week:integer read GetWeek write SetWeek;
    property FirstWeek : integer read GetFirstWeek write SetFirstWeek;
    property LastWeek : integer read GetLastWeek write SetLastWeek;
  end;


var
  FormTimeTables: TFormTimeTables;

implementation
{$R *.DFM}
uses kstring, kdouble,SubjectsForm,Clipbrd, jpeg;

const TIMECOUNT = 6;
      DATECOUNT = 5;

var dates : array[1..DATECOUNT] of string = ('ma','ti','ke','to','pe');
var times : array[1..TIMECOUNT] of string = ('0810','1012','1214','1416','1618','1820');

//------------------------------------------------------------------------------
// Helpper functions

function DateIndex(const s:string):integer;
var i:integer;
begin
  Result := 0;
  for i:=1 to DATECOUNT do
    if ( dates[i] = s ) then begin Result := i; exit; end;
end;

function TimeIndex(const s:string):integer;
var i:integer;
begin
  Result := 0;
  for i:=1 to TIMECOUNT do
    if ( times[i] = s ) then begin Result := i; exit; end;
end;

procedure SetNames(grid:TStringGrid);
var i:integer;
begin
  for i:=1 to DATECOUNT do grid.Cells[i,0] := dates[i];
  for i:=1 to TIMECOUNT do grid.Cells[0,i] := Copy(times[i],1,2) + ' - ' + Copy(times[i],3,2);
end;

function CGIParse(const s:string):string;
var n,i:integer; hex:string[2];
begin
  Result := '';
  i := 1;
  hex := '00';
  while ( i <= Length(s) ) do begin
    if ( s[i] = '+') then begin Result := Result + ' '; inc(i); continue; end;
    if ( s[i] <> '%') then begin Result := Result + s[i]; inc(i); continue; end;
    if ( i + 2 >= Length(s) ) then exit;
    hex[1] := s[i+1];
    hex[2] := s[i+2];
    n := StrToIntDef('$'+hex,0);
    Result := Result + Char(n);
    Inc(i,3);
  end;
end;

function TakeEmail(var s: string; var lmail:string) : string;
var smail,name,part:string;
begin
  s := CGIParse(s);
  part := Separate(s,'&');
  name := Separate(part,'=');
  Result := '';
  if ( name = 'email') then begin
    lmail := part;
    smail := Separate(part,'@');
    Result := smail;
  end;
end;



procedure ReadUnixln(var f:TextFile; var s:string);
var c:char;
begin
  s := '';
  while not eof(f) do begin
    Read(f,c);
    if c = #10 then exit;
    if c <> #13 then s := s + c;
  end;
end;


{ TTimeTable }

procedure TTimeTable.SetAsString(const Value: string);
var s,mail,name,part:string; d,t:integer; WriteWeek : integer;
begin
  s := Value;
  Caption := TakeEmail(s,mail);
  Email := mail;
  WriteWeek := 0;

  while s <> '' do begin
    part := Separate(s,'&');
    name := Separate(part,'=');
    if ( name = 'vko' ) then begin
      WriteWeek := StrToIntDef(part,0);
      Week := WriteWeek;
      Continue;
    end;

    if ( WriteWeek <> Week ) then continue;
    d := DateIndex(Copy(name,1,2));
    t := TimeIndex(Copy(name,3,100));
    if ( d = 0 ) or ( t = 0 ) then continue;
    Cells[Week,d,t] := Part;
  end;
  Rest.Caption := part;
end;

constructor TTimeTable.Create(AOwner: TComponent);
begin
  inherited;
  PanelText  := TPanel.Create(self);
  FEmail     := TEdit.Create(self);
  Rest       := TLabel.Create(self);
  FWeekPages := TPageControl.Create(self);
  
  with PanelText do begin
    Parent := self;
    Height := 41;
    Align := alTop;
    BevelInner := bvLowered;
    Caption := ' ';
  end;
  with FEmail do begin
    Parent := PanelText;
    Left := 8;
    Top := 8;
    Width := 100;
    Height := 13;
    Text := '';
    ReadOnly := true;
  end;
  with Rest do begin
    Parent := PanelText;
    Left := 200;
    Top := 8;
    Width := 100;
    Height := 13;
    Caption := '';
  end;

  with FWeekPages do begin
    Parent := self;
    Align := alClient;
  end;

end;

constructor TTimeTable.Create(AOwner: TComponent; pg: TPageControl; fw,
  lw: integer);
begin
  Create(AOwner);
  PageControl := pg;
  FirstWeek := fw;
  LastWeek := lw;
end;


procedure TTimeTable.SetFirstWeek(const Value: integer);
var i:integer;
begin
  FFirstWeek := Value;
  for i:=FWeekPages.PageCount-1 downto 0 do
    FWeekPages.Pages[i].Free;

end;

procedure TTimeTable.SetLastWeek(const Value: integer);
var w:integer; wt : TOneWeekTable;
begin
  FLastWeek := Value;
  for w:=FirstWeek to LastWeek do begin
    wt := TOneWeekTable.Create(self);
    wt.PageControl := FWeekPages;
    wt.Name := 'Week'+IntToStr(w);
    wt.Caption := IntToStr(w);
    wt.Grid.OnBeforeDraw := OnBeforeDraw;
    wt.Grid.OnDblClick := OnDblClick;
  end;
end;

function TTimeTable.GetCells(w, c, r: integer): string;
begin
  Result := '';
  if ( not Valid[w] ) then exit;
  Result := Weeks[w].Cells[c,r];

end;

function TTimeTable.GetWeekEmpty(w: integer): boolean;
begin
  Result := false;;
  if ( not Valid[w] ) then exit;
  Result := Weeks[w].Empty;
end;

procedure TTimeTable.SetWeekEmpty(w: integer; const Value: boolean);
begin
  if ( not Valid[w] ) then exit;
  Weeks[w].Empty := Value;
end;


function TTimeTable.GetWeek: integer;
begin
  Result := FWeekPages.ActivePageIndex + FirstWeek;
end;

procedure TTimeTable.SetWeek(const Value: integer);
begin
  if ( not Valid[Value] ) then exit;
  FWeekPages.ActivePageIndex := Value-FirstWeek;
end;

function TTimeTable.GetValid(w: integer): boolean;
begin
  Result := false;;
  if ( w < FirstWeek ) then exit;
  if ( w > LastWeek ) then exit;
  Result := true;
end;

procedure TTimeTable.SetCells(w, c, r: integer; const Value: string);
begin
  if ( not Valid[w] ) then exit;
  Weeks[w].Cells[c,r] := Value;
end;

function TTimeTable.GetCol: integer;
begin
  Result := Weeks[Week].Col
end;

function TTimeTable.GetRow: integer;
begin
  Result := Weeks[Week].Row;
end;

procedure TTimeTable.SetCol(const Value: integer);
begin
  Weeks[Week].Col := Value;
end;

procedure TTimeTable.SetRow(const Value: integer);
begin
  Weeks[Week].Row := Value;
end;

function TTimeTable.GetWeeks(w: integer): TOneWeekTable;
begin
  Result := nil;
  if ( FWeekPages.PageCount = 0 ) then exit;
  if ( not Valid[w] ) then w := FirstWeek;
  Result := TOneWeekTable(FWeekPages.Pages[w-FirstWeek]);
end;

procedure TTimeTable.SetOnBeforeDraw(const Value: TDrawCellEvent);
var w:integer;
begin
  FOnBeforeDraw := Value;
  for w:=FirstWeek to LastWeek do
    Weeks[w].Grid.OnBeforeDraw := OnBeforeDraw;
end;

procedure TTimeTable.SetOnDblClick(const Value: TNotifyEvent);
var w:integer;
begin
  FOnDblClick := Value;
  for w:=FirstWeek to LastWeek do
    Weeks[w].Grid.OnDblClick := OnDblClick;
end;

procedure TTimeTable.SetOnChange(const Value: TNotifyEvent);
begin
  FOnChange := Value;
  FWeekPages.OnChange := Value;
end;


function TTimeTable.GetAsInteger(w, c, r: integer): integer;
begin
  Result := StrToIntDef(Cells[w,c,r],0);
end;

procedure TTimeTable.SetAsInteger(w, c, r: integer; const Value: integer);
begin
  Cells[w,c,r] := IntToStr(Value);
end;

function TTimeTable.GetEMail: string;
begin
  Result := FEMail.Text;
end;

procedure TTimeTable.SetEMail(const Value: string);
begin
  FEMail.Text := Value;
end;

{ TOneWeekTable }


constructor TOneWeekTable.Create(AOwner: TComponent);
begin
  inherited;
  FEmpty := true;
  Grid       := TStringGridIni.Create(self);
  with Grid do begin
    Parent := self;
    Align := alClient;
    ColCount := DATECOUNT+1;
    DefaultColWidth := 120;
    DefaultRowHeight := 50;
    RowCount := TIMECOUNT+1;
  end;
  SetNames(grid);

end;

function TOneWeekTable.GetCells(c, r: integer): string;
begin
  Result := Grid.Cells[c,r];
end;

function TOneWeekTable.GetCol: integer;
begin
  Result := Grid.Col;
end;

function TOneWeekTable.GetRow: integer;
begin
  Result := Grid.Row;
end;

procedure TOneWeekTable.SetAsString(const Value: string);
begin

end;


procedure TOneWeekTable.SetCells(c, r: integer; const Value: string);
begin
  Grid.Cells[c,r] := Value;
  if ( Value <> '' ) then FEmpty := false;
end;

procedure TOneWeekTable.SetCol(const Value: integer);
begin
  Grid.Col := Value;
end;

procedure TOneWeekTable.SetEmpty(const Value: boolean);
var c,r:integer;
begin
  FEmpty := Value;
  for c := 1 to Grid.ColCount do
    for r := 1 to Grid.RowCount do
      Grid.Cells[c,r] := '';
end;

procedure TOneWeekTable.SetRow(const Value: integer);
begin
  Grid.Row := Value;
end;

//------------------------------------------------------------------------------
//

{ TFormTimeTables }

procedure TFormTimeTables.CountTimes;
var tab,r,c,w : integer; tt : TTimeTable;
begin
  for w := FirstWeek to LastWeek do TabAll.WeekEmpty[w] := True;

  for tab := 1 to PGTimes.PageCount-1 do begin
    if not ( PGTimes.Pages[tab] is TTimeTable ) then continue;
    tt := TTimeTable(PGTimes.Pages[tab]);
    for w := FirstWeek to LastWeek do begin
      if tt.WeekEmpty[w] then continue;
      for r:=1 to TIMECOUNT do begin
        for c:=1 to DATECOUNT do begin
          if ( tt.Cells[w,c,r] <> '' ) then
            TabAll.AsInteger[w,c,r] := TabAll.AsInteger[w,c,r] + 1;
        end; // c
      end; // r
    end; // w
  end; // tab
  CounterAnswers.Value := PGTimes.PageCount-1;
end;

function TFormTimeTables.FindTable(const s:string):TTimeTable;
var email,lmail,st:string; tab:integer; tt : TTimeTable;
begin
  Result := nil;
  st := s;
  email := TakeEmail(st,lmail);
  if ( email < ' ' ) then exit;
  email := UpperCase(email);
  for tab := 1 to PGTimes.PageCount-1 do begin
     if not ( PGTimes.Pages[tab] is TTimeTable ) then continue;
     tt := TTimeTable(PGTimes.Pages[tab]);
     if ( UpperCase(tt.Caption) = email ) then begin
       Result := tt;
       exit;
     end;
  end;
end;

function TFormTimeTables.StartsWith(const word,s:string) : boolean;
begin
  Result := false;
  if ( Pos(UpperCase(word),UpperCase(s)) = 0 ) then exit;
  Result := true;
end;

function TFormTimeTables.IsWeeks(word,s:string) : boolean;
begin
  Result := StartsWith(word,s);  if ( not Result ) then exit;
  if not CBReadWeeks.Checked then exit;

  Delete(s,1,Length(Word));
  FirstWeek := ExtractInt(s,'-',0);
  LastWeek := ExtractInt(s,'-',FirstWeek);
end;


procedure TFormTimeTables.ReadFile;
var table : TTimeTable;
    f : TextFile;
    s : string;
begin
  TSAllHold.Free; TSAllHold := nil;    // Kill the place holder
  TabAll.Free; TabAll := nil;          // Kill previous totals

  ClearAll;

  if ( CBReadWeeks.Checked ) then begin
    FirstWeek := 0;
    LastWeek := 0;
  end;

  TabAll := TTimeTable.Create(self,PGTimes,FirstWeek,LastWeek);
  TabAll.OnBeforeDraw := GridBeforeDraw;
  TabAll.OnDblClick := GridDblClick;
  TabAll.OnChange := WeekPagesChange;
  TabAll.Caption := 'All';
  TabAll.eMail := 'All';

  AssignFile(f,EditFileName.Text);
  try
    Reset(f);
    while not eof(f) do begin
      ReadUnixln(f,s);
      if s <= ' ' then continue;
      if ( IsWeeks('Weeks=',s) ) then continue;
      if not ( StartsWith('email=',s) ) then continue;

      table := FindTable(s);
      if ( table = nil ) then 
        table := TTimeTable.Create(self,PGTimes,FirstWeek,LastWeek);
      table.AsString := s;
    end;
    CloseFile(f);
  finally
  end;
  CountTimes;
end;

procedure TFormTimeTables.ButtonReadClick(Sender: TObject);
begin
  ReadFile;
end;

procedure TFormTimeTables.GridBeforeDraw(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var i,black:integer;
begin
  if ( ACol = 0 ) or ( ARow = 0 ) then exit;
  black := ParamBlackLimit.AsInteger;
  with TStringGrid(sender) do begin
    i := StrToIntDef(Cells[ACol,ARow],0);
    i := 255 - ( i * 256 ) div black;
    if ( i < 0 ) then i := 0;
    Canvas.Brush.Color := i + (i*256)+(i*256*256);
  end;
end;

procedure TFormTimeTables.ParamBlackLimitAfterChange(Sender: TObject;
  var val: Double);
begin
  Hide; Show;
end;

procedure TFormTimeTables.GridDblClick(Sender: TObject);
var tab,r,c : integer; tt : TTimeTable; Grid : TStringGrid;
begin
  FormSubjects.Show;
  FormSubjects.Clear;
  if not ( sender is TStringGrid ) then exit;
  Grid := TStringGrid(sender);
  c := Grid.Col;
  r := Grid.Row;
  if ( c < 1 ) or ( r < 1 ) then exit;

  for tab := 1 to PGTimes.PageCount-1 do begin
     if not ( PGTimes.Pages[tab] is TTimeTable ) then continue;
     tt := TTimeTable(PGTimes.Pages[tab]);
     if ( tt.Cells[week,c,r] <> '' ) then begin
       FormSubjects.Add(dates[c]+times[r],tab,tt.Cells[week,c,r]);
       tt.Week := week;
       tt.Col := c;
       tt.Row := r;
     end;
  end;
end;


procedure TFormTimeTables.Choose(i: integer);
begin
  PGTimes.ActivePageIndex := i;
end;

procedure TFormTimeTables.ButtonOKClick(Sender: TObject);
begin
  Close;
end;

procedure TFormTimeTables.ButtonAllClick(Sender: TObject);
begin
  PGTimes.ActivePageIndex := 0;
end;

function TFormTimeTables.GetWeek: integer;
begin
  Result := TabAll.Week;
end;

function TFormTimeTables.GetFirstWeek: integer;
begin
  Result := ParamFirstWeek.AsInteger;
end;

function TFormTimeTables.GetLastWeek: integer;
begin
  Result := ParamLastWeek.AsInteger;
end;

procedure TFormTimeTables.SetFirstWeek(const Value: integer);
begin
  ParamFirstWeek.AsInteger := Value;
  if ( TabAll = nil ) then exit;
  TabAll.FirstWeek := Value;
end;

procedure TFormTimeTables.SetLastWeek(const Value: integer);
begin
  ParamLastWeek.AsInteger := Value;
  if ( TabAll = nil ) then exit;
  TabAll.LastWeek := Value;
end;

procedure TFormTimeTables.WeekPagesChange(Sender: TObject);
var tab : integer; tt : TTimeTable;
begin
  for tab := 1 to PGTimes.PageCount-1 do begin
    if not ( PGTimes.Pages[tab] is TTimeTable ) then continue;
    tt := TTimeTable(PGTimes.Pages[tab]);
    tt.Week := week;
  end;
end;


procedure TFormTimeTables.ClearAll;
var tab : integer; 
begin
  for tab := PGTimes.PageCount-1 downto 0 do begin
    PGTimes.Pages[tab].Free;
  end;
end;

procedure TFormTimeTables.SetWeek(const Value: integer);
begin
  TabAll.Week := Value;
end;

function TFormTimeTables.GetImageBitmap : TBitmap;
var bmp : TBitmap; y,h,w:integer;
begin
  Result := nil;
  if ( TabAll = nil ) then exit;
  bmp := TBitmap.Create;
  if ( bmp = nil ) then exit;
  h := TabAll.Weeks[week].PageControl.Height;
  bmp.Width := TabAll.Weeks[week].PageControl.Width;
  bmp.Height := h * ( LastWeek-FirstWeek +1 );
  y:= 0;
  for w := FirstWeek to LastWeek do begin
    week := w;
    TabAll.Weeks[w].PageControl.PaintTo(bmp.Canvas.Handle,0,y);
    y := y + h;
  end;
  Result := bmp;
end;


procedure TFormTimeTables.ActionEditCopyExecute(Sender: TObject);
var bmp : TBitmap;
begin
  bmp := GetImageBitmap; if ( bmp = nil ) then exit;
  Clipboard.Assign(bmp);
  bmp.Free;
end;

procedure TFormTimeTables.ActionFileSaveExecute(Sender: TObject);
var bmp : TBitmap;
    jp: TJpegImage;  //Requires the "jpeg" unit added to "uses" clause.
begin
  if ( FImageName = '' ) then begin
    if not SavePictureDialog1.Execute then exit;
    FImageName := SavePictureDialog1.FileName;
    if ( FImageName = '' ) then exit;
  end;
  bmp := GetImageBitmap; if ( bmp = nil ) then exit;
  jp := TJpegImage.Create;
  jp.Assign(bmp);
  jp.SaveToFile(FImageName);
  jp.Free;
  bmp.Free;
end;

procedure TFormTimeTables.ActionFileExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormTimeTables.ActionFileOpenExecute(Sender: TObject);
begin
  OpenDialog1.FileName := EditFileName.Text;
  if ( not OpenDialog1.Execute ) then exit;
  EditFileName.Text := OpenDialog1.FileName;
  ButtonReadClick(self);
end;

//------------------------------------------------------------------------------
// Code for automation
//

function TFormTimeTables.StepOPEN(const s:string):string;
begin
  EditFileName.Text := s;
  ReadFile;
  Result := '';
end;

function TFormTimeTables.StepSAVEAS(const s:string):string;
begin
  FImageName := s;
  ActionFileSaveExecute(self);
  Result := '';
end;

function TFormTimeTables.StepEXEC(const s:string):string;
begin
  ReadFile;
  Result := '';
end;

function TFormTimeTables.StepCOPY(const s:string):string;
begin
  ActionEditCopyExecute(self);
  Result := '';
end;

function TFormTimeTables.StepCLOSE(const s:string):string;
begin
  Close;
  Result := '';
end;


procedure TFormTimeTables.FormCreate(Sender: TObject);
begin
  SetNames(Grid);

  Programs := TParamProgram.Create(self);
  Programs.Add(TProgramCommand.Create('OPEN',StepOPEN,'datafile'));
  Programs.Add(TProgramCommand.Create('SAVEAS',StepSAVEAS,'JPG_filename'));
  Programs.Add(TProgramCommand.Create('EXEC',StepEXEC,''));
  Programs.Add(TProgramCommand.Create('COPY',StepCOPY));
  Programs.Add(TProgramCommand.Create('CLOSE',StepCLOSE));
//  Programs.Title := LabelProgram;
//  Programs.ParamCtr := ScrollBoxParams;
  Programs.EditWidth := 150;
  Programs.EditHeight := 20;
end;


function TFormTimeTables.CheckProgram : boolean;
var s:string;
begin
  Result := false;
  if ParamCount <= 0 then exit;

  if ParamCount <= 1 then begin
    EditFileName.Text := ParamStr(1);
    ReadFile;
  end;

  if ( ParamStr(1) <> 'RUN' ) and ( ParamStr(1) <> 'LOAD' ) then exit;

  s := Programs.Read(ParamStr(2));
  if ( s <> '' ) then begin ShowMessage(s); exit; end;

  if ( ParamStr(1) = 'RUN' ) then Programs.Run;
  Result := true;

end;

procedure TFormTimeTables.ActionShowProgramCommandsExecute(
  Sender: TObject);
begin
  Programs.ShowProgramCommands;
end;

end.
